﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Interactivity;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;
using Microsoft.Practices.Prism.Regions;

namespace UI.Core.Interaction
{
    public class PopupWindowAction : TriggerAction<FrameworkElement>
    {
        public static readonly DependencyProperty WindowContentProperty = DependencyProperty.Register("WindowContent", typeof(FrameworkElement), typeof(PopupWindowAction), new PropertyMetadata(null));
        public static readonly DependencyProperty ContentTemplateProperty = DependencyProperty.Register("ContentTemplate", typeof(DataTemplate), typeof(PopupWindowAction), new PropertyMetadata(null));
        public static readonly DependencyProperty IsModalProperty = DependencyProperty.Register("IsModal", typeof(bool), typeof(PopupWindowAction), new PropertyMetadata(null));
        public static readonly DependencyProperty CenterOverAssociatedObjectProperty = DependencyProperty.Register("CenterOverAssociatedObject", typeof(bool), typeof(PopupWindowAction), new PropertyMetadata(null));

        public FrameworkElement WindowContent
        {
            get { return (FrameworkElement)GetValue(WindowContentProperty); }
            set { SetValue(WindowContentProperty, value); }
        }
        public DataTemplate ContentTemplate
        {
            get { return (DataTemplate)GetValue(ContentTemplateProperty); }
            set { SetValue(ContentTemplateProperty, value); }
        }
        public bool IsModal
        {
            get { return (bool)GetValue(IsModalProperty); }
            set { SetValue(IsModalProperty, value); }
        }
        public bool CenterOverAssociatedObject
        {
            get { return (bool)GetValue(CenterOverAssociatedObjectProperty); }
            set { SetValue(CenterOverAssociatedObjectProperty, value); }
        }
        protected override void Invoke(object parameter)
        {
            var args = parameter as InteractionRequestedEventArgs;
            if (args == null)
            {
                return;
            }

            // If the WindowContent shouldn't be part of another visual tree.
            if (this.WindowContent != null && this.WindowContent.Parent != null)
            {
                return;
            }

            var wrapperWindow = this.GetWindow(args.Context);

            var callback = args.Callback;
            EventHandler handler = null;
            handler =
                (o, e) =>
                {
                    wrapperWindow.Closed -= handler;
                    wrapperWindow.Content = null;
                    callback();
                };
            wrapperWindow.Closed += handler;

            if (this.CenterOverAssociatedObject)
            {
                SizeChangedEventHandler sizeHandler = null;
                sizeHandler =
                    (o, e) =>
                    {
                        wrapperWindow.SizeChanged -= sizeHandler;

                        FrameworkElement invoker = this.AssociatedObject;
                        Point position = invoker.PointToScreen(new Point(0, 0));

                        wrapperWindow.Top = position.Y + ((invoker.ActualHeight - wrapperWindow.ActualHeight) / 2);
                        wrapperWindow.Left = position.X + ((invoker.ActualWidth - wrapperWindow.ActualWidth) / 2);
                    };
                wrapperWindow.SizeChanged += sizeHandler;
            }

            if (this.IsModal)
            {
                wrapperWindow.ShowDialog();
            }
            else
            {
                wrapperWindow.Show();
            }
        }
        protected void PrepareContentForWindow(Notification notification, Window wrapperWindow)
        {
            if (this.WindowContent == null)
            {
                return;
            }

            // We set the WindowContent as the content of the window. 
            wrapperWindow.Content = this.WindowContent;

            IRegionManager regionManager =
                this.WindowContent.GetValue(RegionManager.RegionManagerProperty) as IRegionManager;

            // If the WindowContent does not have a RegionManager attached we create a new scoped RegionManager for it in order to support regions.
            if (regionManager == null)
            {
                regionManager = new RegionManager();
                this.WindowContent.SetValue(RegionManager.RegionManagerProperty, regionManager);
            }

            // If the WindowContent implements IRegionManagerAware we set the new scoped manager as the RegionManager.
            IRegionManagerAware regionManagerAwareContent = this.WindowContent as IRegionManagerAware;
            if (regionManagerAwareContent != null)
            {
                regionManagerAwareContent.RegionManager = regionManager;
            }

            // If the WindowContent's DataContext implements IRegionManagerAware we set the new scoped manager as the RegionManager.
            IRegionManagerAware regionManagerAwareDataContext = this.WindowContent.DataContext as IRegionManagerAware;
            if (regionManagerAwareDataContext != null)
            {
                regionManagerAwareDataContext.RegionManager = regionManager;
            }

            // If the WindowContent implements IPopupWindowActionAware, we set the corresponding values.
            IPopupWindowActionAware popupAwareContent = this.WindowContent as IPopupWindowActionAware;
            if (popupAwareContent != null)
            {
                popupAwareContent.HostWindow = wrapperWindow;
                popupAwareContent.HostNotification = notification;
            }

            // If the WindowContent's DataContext implements IPopupWindowActionAware, we set the corresponding values.
            IPopupWindowActionAware popupAwareDataContext = this.WindowContent.DataContext as IPopupWindowActionAware;
            if (popupAwareDataContext != null)
            {
                popupAwareDataContext.HostWindow = wrapperWindow;
                popupAwareDataContext.HostNotification = notification;
            }
        }
        protected Window GetWindow(Notification notification)
        {
            Window wrapperWindow;
            wrapperWindow = new Window();

            if (this.WindowContent != null)
            {
                // If the WindowContent does not have its own DataContext, it will inherit this one.
                wrapperWindow.DataContext = notification;
                wrapperWindow.Title = notification.Title;

                wrapperWindow.SizeToContent = SizeToContent.WidthAndHeight;

                this.PrepareContentForWindow(notification, wrapperWindow);
            }
            else
            {
                //wrapperWindow = this.CreateDefaultWindow(notification);
                //wrapperWindow.DataContext = notification;
            }

            return wrapperWindow;
        }

        //private Window CreateDefaultWindow(Notification notification)
        //{
        //    return notification is Confirmation
        //        ? (Window)new DefaultConfirmationWindow { ConfirmationTemplate = this.ContentTemplate }
        //        : new DefaultNotificationWindow { NotificationTemplate = this.ContentTemplate };
        //}
    }
}
